"use strict";

Object.defineProperty(exports, "__esModule", {
  value: true
});
Object.defineProperty(exports, "useThemeProps", {
  enumerable: true,
  get: function () {
    return _useProps.useThemeProps;
  }
});
Object.defineProperty(exports, "usePropsResolution", {
  enumerable: true,
  get: function () {
    return _usePropsResolution.usePropsResolution;
  }
});
exports.useComponentThemePropsResolution = void 0;

var _useProps = require("./useProps");

var _usePropsResolution = require("./usePropsResolution");

var _lodash = _interopRequireDefault(require("lodash.merge"));

var _getStyledComponentAndObjects = require("../../utils/getStyledComponentAndObjects");

var _reactNative = require("react-native");

var _tools = require("../../theme/tools");

var _core = require("../../core");

var _isEmptyObj = require("../../utils/isEmptyObj");

var _NativeBaseContext = require("../../core/NativeBaseContext");

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

const sizesExistsInTheme = (componentTheme, size) => {
  if (componentTheme !== null && componentTheme !== void 0 && componentTheme.sizes) {
    if (componentTheme.sizes[size]) {
      return true;
    }
  }

  return false;
};

const useComponentThemePropsResolution = (componentTheme, incomingProps, theme) => {
  const {
    colorMode
  } = (0, _core.useColorMode)();
  const providerId = (0, _NativeBaseContext.useNativeBaseConfig)('NativeBase').providerId;

  const incomingInternalThemeProps = _lodash.default.apply({}, incomingProps === null || incomingProps === void 0 ? void 0 : incomingProps.INTERNAL_themeStyle);

  const incomingStateProps = incomingProps === null || incomingProps === void 0 ? void 0 : incomingProps.stateProps;
  const modifiedIncomingProps = incomingProps;
  modifiedIncomingProps === null || modifiedIncomingProps === void 0 ? true : delete modifiedIncomingProps.INTERNAL_themeStyle;
  modifiedIncomingProps === null || modifiedIncomingProps === void 0 ? true : delete modifiedIncomingProps.stateProps;
  const styleObj = (0, _getStyledComponentAndObjects.getStyledObject)(theme, '', componentTheme, {
    platform: _reactNative.Platform.OS,
    colorMode: colorMode
  }, (0, _lodash.default)({}, modifiedIncomingProps), true);
  const resolvedPropsWithStateProps = (0, _usePropsResolution.usePropsResolutionWithComponentTheme)(componentTheme, (0, _lodash.default)({}, styleObj === null || styleObj === void 0 ? void 0 : styleObj.unResolvedProps, styleObj === null || styleObj === void 0 ? void 0 : styleObj.restDefaultProps, modifiedIncomingProps), theme, {});
  resolvedPropsWithStateProps.flattenProps = (0, _lodash.default)({}, resolvedPropsWithStateProps.flattenProps, modifiedIncomingProps);
  let resolvedFlattenProps = resolvedPropsWithStateProps.flattenProps;
  const resolvedStateProps = { ...(resolvedPropsWithStateProps === null || resolvedPropsWithStateProps === void 0 ? void 0 : resolvedPropsWithStateProps.stateProps),
    ...incomingStateProps
  };
  const restDefaultWithStateProps = styleObj === null || styleObj === void 0 ? void 0 : styleObj.stateRestDefaultProps;
  const stateStyleFromProps = (0, _tools.omitUndefined)(styleObj === null || styleObj === void 0 ? void 0 : styleObj.stateStyleFromProps);

  for (const property in restDefaultWithStateProps) {
    var _resolvedPropsWithSta;

    resolvedFlattenProps[property] = (_resolvedPropsWithSta = resolvedPropsWithStateProps.stateProps[property]) !== null && _resolvedPropsWithSta !== void 0 ? _resolvedPropsWithSta : restDefaultWithStateProps[property];
  } // Merge default props with inline resolved props


  resolvedFlattenProps.INTERNAL_themeStyle = [styleObj.styleFromProps, incomingInternalThemeProps];
  resolvedStateProps.INTERNAL_themeStyle = (0, _isEmptyObj.isEmptyObj)(stateStyleFromProps) ? [] : [stateStyleFromProps]; // merge rest default props with resolved props

  resolvedFlattenProps = { ...resolvedFlattenProps,
    ...styleObj.restDefaultProps
  };

  if (resolvedFlattenProps.size) {
    if (!sizesExistsInTheme(componentTheme, resolvedFlattenProps.size) && (0, _tools.isLiteral)(resolvedFlattenProps.size)) {
      resolvedFlattenProps = {
        boxSize: resolvedFlattenProps.size,
        ...resolvedFlattenProps
      };
    }

    resolvedFlattenProps.size = undefined;
  }

  for (const property in styleObj === null || styleObj === void 0 ? void 0 : styleObj.internalPseudoProps) {
    if (_core.PSEUDO_PROP_COMPONENT_MAP[property]) {
      var _resolvedFlattenProps;

      const pseudoComponentThemeProps = (0, _core.getThemeProps)(theme, providerId, "", {
        colorMode,
        platform: _reactNative.Platform.OS
      }, {}, incomingProps, false);
      resolvedFlattenProps[property] = { ...pseudoComponentThemeProps.restDefaultProps,
        ...(styleObj === null || styleObj === void 0 ? void 0 : styleObj.internalPseudoProps[property]),
        ...resolvedFlattenProps[property],
        INTERNAL_themeStyle: (_resolvedFlattenProps = resolvedFlattenProps[property]) !== null && _resolvedFlattenProps !== void 0 && _resolvedFlattenProps.INTERNAL_themeStyle ? [pseudoComponentThemeProps.styleFromProps, ...resolvedFlattenProps[property].INTERNAL_themeStyle] : [pseudoComponentThemeProps.styleFromProps]
      };
    }
  } // for inline pseudo compoennt props- merge property


  for (const property in resolvedStateProps) {
    if (_core.PSEUDO_PROP_COMPONENT_MAP[property]) {
      if (resolvedFlattenProps[property]) {
        resolvedFlattenProps[property].stateProps = resolvedStateProps[property];
      } else {
        resolvedFlattenProps[property] = {
          stateProps: resolvedStateProps[property]
        };
      }
    }
  }

  const resolvedProps = (0, _tools.omitUndefined)({ ...resolvedFlattenProps,
    stateProps: resolvedStateProps
  });
  return resolvedProps;
};

exports.useComponentThemePropsResolution = useComponentThemePropsResolution;
//# sourceMappingURL=index.js.map