export { useThemeProps } from './useProps';
export { usePropsResolution } from './usePropsResolution';
import { usePropsResolutionWithComponentTheme } from './usePropsResolution';
import merge from 'lodash.merge';
import { getStyledObject } from '../../utils/getStyledComponentAndObjects';
import { Platform } from 'react-native';
import { isLiteral, omitUndefined } from '../../theme/tools';
import { PSEUDO_PROP_COMPONENT_MAP, useColorMode, getThemeProps } from '../../core';
import { isEmptyObj } from '../../utils/isEmptyObj';
import { useNativeBaseConfig } from '../../core/NativeBaseContext';

const sizesExistsInTheme = (componentTheme, size) => {
  if (componentTheme !== null && componentTheme !== void 0 && componentTheme.sizes) {
    if (componentTheme.sizes[size]) {
      return true;
    }
  }

  return false;
};

export const useComponentThemePropsResolution = (componentTheme, incomingProps, theme) => {
  const {
    colorMode
  } = useColorMode();
  const providerId = useNativeBaseConfig('NativeBase').providerId;
  const incomingInternalThemeProps = merge.apply({}, incomingProps === null || incomingProps === void 0 ? void 0 : incomingProps.INTERNAL_themeStyle);
  const incomingStateProps = incomingProps === null || incomingProps === void 0 ? void 0 : incomingProps.stateProps;
  const modifiedIncomingProps = incomingProps;
  modifiedIncomingProps === null || modifiedIncomingProps === void 0 ? true : delete modifiedIncomingProps.INTERNAL_themeStyle;
  modifiedIncomingProps === null || modifiedIncomingProps === void 0 ? true : delete modifiedIncomingProps.stateProps;
  const styleObj = getStyledObject(theme, '', componentTheme, {
    platform: Platform.OS,
    colorMode: colorMode
  }, merge({}, modifiedIncomingProps), true);
  const resolvedPropsWithStateProps = usePropsResolutionWithComponentTheme(componentTheme, merge({}, styleObj === null || styleObj === void 0 ? void 0 : styleObj.unResolvedProps, styleObj === null || styleObj === void 0 ? void 0 : styleObj.restDefaultProps, modifiedIncomingProps), theme, {});
  resolvedPropsWithStateProps.flattenProps = merge({}, resolvedPropsWithStateProps.flattenProps, modifiedIncomingProps);
  let resolvedFlattenProps = resolvedPropsWithStateProps.flattenProps;
  const resolvedStateProps = { ...(resolvedPropsWithStateProps === null || resolvedPropsWithStateProps === void 0 ? void 0 : resolvedPropsWithStateProps.stateProps),
    ...incomingStateProps
  };
  const restDefaultWithStateProps = styleObj === null || styleObj === void 0 ? void 0 : styleObj.stateRestDefaultProps;
  const stateStyleFromProps = omitUndefined(styleObj === null || styleObj === void 0 ? void 0 : styleObj.stateStyleFromProps);

  for (const property in restDefaultWithStateProps) {
    var _resolvedPropsWithSta;

    resolvedFlattenProps[property] = (_resolvedPropsWithSta = resolvedPropsWithStateProps.stateProps[property]) !== null && _resolvedPropsWithSta !== void 0 ? _resolvedPropsWithSta : restDefaultWithStateProps[property];
  } // Merge default props with inline resolved props


  resolvedFlattenProps.INTERNAL_themeStyle = [styleObj.styleFromProps, incomingInternalThemeProps];
  resolvedStateProps.INTERNAL_themeStyle = isEmptyObj(stateStyleFromProps) ? [] : [stateStyleFromProps]; // merge rest default props with resolved props

  resolvedFlattenProps = { ...resolvedFlattenProps,
    ...styleObj.restDefaultProps
  };

  if (resolvedFlattenProps.size) {
    if (!sizesExistsInTheme(componentTheme, resolvedFlattenProps.size) && isLiteral(resolvedFlattenProps.size)) {
      resolvedFlattenProps = {
        boxSize: resolvedFlattenProps.size,
        ...resolvedFlattenProps
      };
    }

    resolvedFlattenProps.size = undefined;
  }

  for (const property in styleObj === null || styleObj === void 0 ? void 0 : styleObj.internalPseudoProps) {
    if (PSEUDO_PROP_COMPONENT_MAP[property]) {
      var _resolvedFlattenProps;

      const pseudoComponentThemeProps = getThemeProps(theme, providerId, "", {
        colorMode,
        platform: Platform.OS
      }, {}, incomingProps, false);
      resolvedFlattenProps[property] = { ...pseudoComponentThemeProps.restDefaultProps,
        ...(styleObj === null || styleObj === void 0 ? void 0 : styleObj.internalPseudoProps[property]),
        ...resolvedFlattenProps[property],
        INTERNAL_themeStyle: (_resolvedFlattenProps = resolvedFlattenProps[property]) !== null && _resolvedFlattenProps !== void 0 && _resolvedFlattenProps.INTERNAL_themeStyle ? [pseudoComponentThemeProps.styleFromProps, ...resolvedFlattenProps[property].INTERNAL_themeStyle] : [pseudoComponentThemeProps.styleFromProps]
      };
    }
  } // for inline pseudo compoennt props- merge property


  for (const property in resolvedStateProps) {
    if (PSEUDO_PROP_COMPONENT_MAP[property]) {
      if (resolvedFlattenProps[property]) {
        resolvedFlattenProps[property].stateProps = resolvedStateProps[property];
      } else {
        resolvedFlattenProps[property] = {
          stateProps: resolvedStateProps[property]
        };
      }
    }
  }

  const resolvedProps = omitUndefined({ ...resolvedFlattenProps,
    stateProps: resolvedStateProps
  });
  return resolvedProps;
};
//# sourceMappingURL=index.js.map